@@ -97,7 +97,7 @@ class XEP_0325(BasePlugin):
 
     name = 'xep_0325'
     description = 'XEP-0325 Internet of Things - Control'
-    dependencies = set(['xep_0030'])
+    dependencies = {'xep_0030'}
     stanza = stanza
 
 
@@ -540,7 +540,7 @@ class XEP_0325(BasePlugin):
         fields = [f['name'] for f in iq['setResponse']['datas']]
         error_msg = None
 
-        if not iq['setResponse'].find('error') is None and not iq['setResponse']['error']['text'] == "":
+        if not iq['setResponse'].xml.find('error') is None and not iq['setResponse']['error']['text'] == "":
             error_msg = iq['setResponse']['error']['text']
 
         callback = self.sessions[seqnr]["callback"]
