@@ -19,10 +19,10 @@ import ssl
 import weakref
 import uuid
 
-import xml.etree.ElementTree
+import xml.etree.ElementTree as ET
 
 from slixmpp.xmlstream.asyncio import asyncio
-from slixmpp.xmlstream import tostring, highlight
+from slixmpp.xmlstream import tostring
 from slixmpp.xmlstream.stanzabase import StanzaBase, ElementBase
 from slixmpp.xmlstream.resolver import resolve, default_resolver
 
@@ -204,6 +204,9 @@ class XMLStream(asyncio.BaseProtocol):
         #: We use an ID prefix to ensure that all ID values are unique.
         self._id_prefix = '%s-' % uuid.uuid4()
 
+        # Current connection attempt (Future)
+        self._current_connection_attempt = None
+
         #: A list of DNS results that have not yet been tried.
         self.dns_answers = None
 
@@ -256,15 +259,16 @@ class XMLStream(asyncio.BaseProtocol):
         TODO fix the comment
         :param force_starttls: If True, the connection will be aborted if
                                the server does not initiate a STARTTLS
-                               negociation.  If None, the connection will be
+                               negotiation.  If None, the connection will be
                                upgraded to TLS only if the server initiate
-                               the STARTTLS negociation, otherwise it will
+                               the STARTTLS negotiation, otherwise it will
                                connect in clear.  If False it will never
                                upgrade to TLS, even if the server provides
                                it.  Use this for example if you’re on
                                localhost
 
         """
+        self.cancel_connection_attempt()
         if host and port:
             self.address = (host, int(port))
         try:
@@ -281,7 +285,7 @@ class XMLStream(asyncio.BaseProtocol):
             self.disable_starttls = disable_starttls
 
         self.event("connecting")
-        asyncio.async(self._connect_routine())
+        self._current_connection_attempt = asyncio.async(self._connect_routine())
 
     @asyncio.coroutine
     def _connect_routine(self):
@@ -289,7 +293,8 @@ class XMLStream(asyncio.BaseProtocol):
 
         record = yield from self.pick_dns_answer(self.default_domain)
         if record is not None:
-            host, address, port = record
+            host, address, dns_port = record
+            port = dns_port if dns_port else self.address[1]
             self.address = (address, port)
             self._service_name = host
         else:
@@ -297,12 +302,19 @@ class XMLStream(asyncio.BaseProtocol):
             # and try (host, port) as a last resort
             self.dns_answers = None
 
+        if self.use_ssl:
+            ssl_context = self.get_ssl_context()
+        else:
+            ssl_context = None
+
         yield from asyncio.sleep(self.connect_loop_wait)
         try:
             yield from self.loop.create_connection(lambda: self,
                                                    self.address[0],
                                                    self.address[1],
-                                                   ssl=self.use_ssl)
+                                                   ssl=ssl_context,
+                                                   server_hostname=self.default_domain if self.use_ssl else None)
+            self.connect_loop_wait = 0
         except Socket.gaierror as e:
             self.event('connection_failed',
                        'No DNS record available for %s' % self.default_domain)
@@ -310,9 +322,7 @@ class XMLStream(asyncio.BaseProtocol):
             log.debug('Connection failed: %s', e)
             self.event("connection_failed", e)
             self.connect_loop_wait = self.connect_loop_wait * 2 + 1
-            asyncio.async(self._connect_routine())
-        else:
-            self.connect_loop_wait = 0
+            self._current_connection_attempt = asyncio.async(self._connect_routine())
 
     def process(self, *, forever=True, timeout=None):
         """Process all the available XMPP events (receiving or sending data on the
@@ -338,7 +348,7 @@ class XMLStream(asyncio.BaseProtocol):
         """
         self.xml_depth = 0
         self.xml_root = None
-        self.parser = xml.etree.ElementTree.XMLPullParser(("start", "end"))
+        self.parser = ET.XMLPullParser(("start", "end"))
 
     def connection_made(self, transport):
         """Called when the TCP connection has been established with the server
@@ -357,33 +367,50 @@ class XMLStream(asyncio.BaseProtocol):
         event.  This could trigger one or more event (a stanza is received,
         the stream is opened, etc).
         """
+        if self.parser is None:
+            log.warning('Received data before the connection is established: %r',
+                        data)
+            return
         self.parser.feed(data)
-        for event, xml in self.parser.read_events():
-            if event == 'start':
-                if self.xml_depth == 0:
-                    # We have received the start of the root element.
-                    self.xml_root = xml
-                    log.debug('[33;1mRECV[0m: %s', highlight(tostring(self.xml_root, xmlns=self.default_ns,
-                                                         stream=self,
-                                                         top_level=True,
-                                                         open_only=True)))
-                    self.start_stream_handler(self.xml_root)
-                self.xml_depth += 1
-            if event == 'end':
-                self.xml_depth -= 1
-                if self.xml_depth == 0:
-                    # The stream's root element has closed,
-                    # terminating the stream.
-                    log.debug("End of stream received")
-                    self.abort()
-                elif self.xml_depth == 1:
-                    # A stanza is an XML element that is a direct child of
-                    # the root element, hence the check of depth == 1
-                    self.loop.idle_call(functools.partial(self.__spawn_event, xml))
-                    if self.xml_root is not None:
-                        # Keep the root element empty of children to
-                        # save on memory use.
-                        self.xml_root.clear()
+        try:
+            for event, xml in self.parser.read_events():
+                if event == 'start':
+                    if self.xml_depth == 0:
+                        # We have received the start of the root element.
+                        self.xml_root = xml
+                        log.debug('RECV: %s', tostring(self.xml_root,
+                                                       xmlns=self.default_ns,
+                                                       stream=self,
+                                                       top_level=True,
+                                                       open_only=True))
+                        self.start_stream_handler(self.xml_root)
+                    self.xml_depth += 1
+                if event == 'end':
+                    self.xml_depth -= 1
+                    if self.xml_depth == 0:
+                        # The stream's root element has closed,
+                        # terminating the stream.
+                        log.debug("End of stream received")
+                        self.abort()
+                    elif self.xml_depth == 1:
+                        # A stanza is an XML element that is a direct child of
+                        # the root element, hence the check of depth == 1
+                        self._spawn_event(xml)
+                        if self.xml_root is not None:
+                            # Keep the root element empty of children to
+                            # save on memory use.
+                            self.xml_root.clear()
+        except ET.ParseError:
+            log.error('Parse error: %r', data)
+
+            # Due to cyclic dependencies, this can’t be imported at the module
+            # level.
+            from slixmpp.stanza.stream_error import StreamError
+            error = StreamError()
+            error['condition'] = 'not-well-formed'
+            error['text'] = 'Server sent: %r' % data
+            self.send(error)
+            self.disconnect()
 
     def is_connected(self):
         return self.transport is not None
@@ -407,6 +434,17 @@ class XMLStream(asyncio.BaseProtocol):
         self.transport = None
         self.socket = None
 
+    def cancel_connection_attempt(self):
+        """
+        Immediatly cancel the current create_connection() Future.
+        This is useful when a client using slixmpp tries to connect
+        on flaky networks, where sometimes a connection just gets lost
+        and it needs to reconnect while the attempt is still ongoing.
+        """
+        if self._current_connection_attempt:
+            self._current_connection_attempt.cancel()
+            self._current_connection_attempt = None
+
     def disconnect(self, wait=2.0):
         """Close the XML stream and wait for an acknowldgement from the server for
         at most `wait` seconds.  After the given number of seconds has
@@ -420,6 +458,7 @@ class XMLStream(asyncio.BaseProtocol):
         :param wait: Time to wait for a response from the server.
 
         """
+        self.cancel_connection_attempt()
         if self.transport:
             self.send_raw(self.stream_footer)
             self.schedule('Disconnect wait', wait,
@@ -429,6 +468,7 @@ class XMLStream(asyncio.BaseProtocol):
         """
         Forcibly close the connection
         """
+        self.cancel_connection_attempt()
         if self.transport:
             self.transport.close()
             self.transport.abort()
@@ -468,14 +508,10 @@ class XMLStream(asyncio.BaseProtocol):
         """
         pass
 
-    def start_tls(self):
-        """Perform handshakes for TLS.
-
-        If the handshake is successful, the XML stream will need
-        to be restarted.
+    def get_ssl_context(self):
+        """
+        Get SSL context.
         """
-        self.event_when_connected = "tls_success"
-
         if self.ciphers is not None:
             self.ssl_context.set_ciphers(self.ciphers)
         if self.keyfile and self.certfile:
@@ -490,7 +526,18 @@ class XMLStream(asyncio.BaseProtocol):
             self.ssl_context.verify_mode = ssl.CERT_REQUIRED
             self.ssl_context.load_verify_locations(cafile=self.ca_certs)
 
-        ssl_connect_routine = self.loop.create_connection(lambda: self, ssl=self.ssl_context,
+        return self.ssl_context
+
+    def start_tls(self):
+        """Perform handshakes for TLS.
+
+        If the handshake is successful, the XML stream will need
+        to be restarted.
+        """
+        self.event_when_connected = "tls_success"
+
+        ssl_context = self.get_ssl_context()
+        ssl_connect_routine = self.loop.create_connection(lambda: self, ssl=ssl_context,
                                                           sock=self.socket,
                                                           server_hostname=self.default_domain)
         @asyncio.coroutine
@@ -505,7 +552,9 @@ class XMLStream(asyncio.BaseProtocol):
                 else:
                     self.event('ssl_invalid_chain', e)
             else:
-                der_cert = transp.get_extra_info("socket").getpeercert(True)
+                # Workaround for a regression in 3.4 where ssl_object was not set.
+                der_cert = transp.get_extra_info("ssl_object",
+                                                 default=transp.get_extra_info("socket")).getpeercert(True)
                 pem_cert = ssl.DER_cert_to_PEM_cert(der_cert)
                 self.event('ssl_cert', pem_cert)
 
@@ -759,6 +808,9 @@ class XMLStream(asyncio.BaseProtocol):
         :param repeat: Flag indicating if the scheduled event should
                        be reset and repeat after executing.
         """
+        if name in self.scheduled_events:
+            raise ValueError(
+                "There is already a scheduled event of name: %s" % name)
         if seconds is None:
             seconds = RESPONSE_TIMEOUT
         cb = functools.partial(callback, *args, **kwargs)
@@ -768,7 +820,6 @@ class XMLStream(asyncio.BaseProtocol):
         else:
             handle = self.loop.call_later(seconds, self._execute_and_unschedule,
                                           name, cb)
-
         # Save that handle, so we can just cancel this scheduled event by
         # canceling scheduled_events[name]
         self.scheduled_events[name] = handle
@@ -839,8 +890,7 @@ class XMLStream(asyncio.BaseProtocol):
                     if data is None:
                         return
             str_data = tostring(data.xml, xmlns=self.default_ns,
-                                          stream=self,
-                                          top_level=True)
+                                stream=self, top_level=True)
             self.send_raw(str_data)
         else:
             self.send_raw(data)
@@ -858,7 +908,7 @@ class XMLStream(asyncio.BaseProtocol):
 
         :param string data: Any bytes or utf-8 string value.
         """
-        log.debug("[36;1mSEND[0m: %s", highlight(data))
+        log.debug("SEND: %s", data)
         if not self.transport:
             raise NotConnectedError()
         if isinstance(data, str):
@@ -890,7 +940,7 @@ class XMLStream(asyncio.BaseProtocol):
             stanza['lang'] = self.peer_default_lang
         return stanza
 
-    def __spawn_event(self, xml):
+    def _spawn_event(self, xml):
         """
         Analyze incoming XML stanzas and convert them into stanza
         objects if applicable and queue stream events to be processed
@@ -911,7 +961,7 @@ class XMLStream(asyncio.BaseProtocol):
         if stanza is None:
             return
 
-        log.debug("[33;1mRECV[0m: %s", highlight(stanza))
+        log.debug("RECV: %s", stanza)
 
         # Match the stanza against registered handlers. Handlers marked
         # to run "in stream" will be executed immediately; the rest will
